#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#	Copyright (C) 2008 General de Software de Canarias.
#	Author: Carlos López Pérez <carlos.lopezperez@gmail.com>
#	Translation (and indentation): Rafael Fonseca <rafael.magu@gmail.com>
#

import re, ldap

from genshi.builder import tag
from trac.core import implements, Component
from trac.web.api import IRequestFilter, IRequestHandler
from trac.web.chrome import INavigationContributor, ITemplateProvider

class AccountLDAP(Component):
	#	Prepares the API to interact with LDAP and Trac
	implements(IRequestFilter, INavigationContributor, ITemplateProvider, IRequestHandler)

	MODULE_NAME = 'accountldap'

	def __init__(self):
		#	Connection to LDAP
		self.basedn = self.config.get('ldap', 'basedn')
		self.userdn = self.config.get('ldap', 'user_rdn')
		try:
			self.ldap = ldap.open(self.config.get('ldap', 'host'))
			self.ldap.simple_bind(self.config.get('ldap', 'bind_user'),
			self.config.get('ldap', 'bind_passwd'))
		except ldap.LDAPError, e:
				self.log.error('LDAP connection error. Check your [ldap] options.')
				self.enabled = false
				self.log.info('Connection LDAP - BaseDN "%s" UserDN "%s"' % (self.basedn,self.userdn))

	#	IRequestFilter interface
	def pre_process_request(self, req, handler):
		return handler

	def post_process_request(self, req, template, data, content_type):
		if not req.remote_user or req.session.has_key('email'):
			return template, data, None
		uid = req.remote_user.lower()
		name, email = self._getUserAttributes(uid)
		req.session['name'] = name
		req.session['email'] = email
		return template, data, None

	#	INavigationContributor interface
	def get_active_navigation_item(self, req):
		return self.MODULE_NAME

	def get_navigation_items(self, req):
		if not req.authname or not req.session.has_key('email'):
			return
		yield ('metanav', self.MODULE_NAME, tag.a(u'Passwords', href=req.href.accountldap()))

	#	IRequestHandler interface
	def match_request(self, req):
		return re.match(r'/%s(?:_trac)?(?:/.*)?$' % self.MODULE_NAME, req.path_info)
    
	def process_request(self, req):
		data = {'accountldap_message': None}
		template = '%s.html' % self.MODULE_NAME
		if req.method != 'POST':
			return template, data, None
		p1 = req.args.get('password1')
		p2 = req.args.get('password2')
		old = req.args.get('oldpassword')
		if p1 != p2:
			data['accountldap_message'] = tag.center(u'The passwords provided do not match.', tag.b(u' Please check your entries and try again.'), style='color:chocolate')
			return template, data, None
		if old == p1:
			data['accountldap_message'] = tag.center(u'You cannot use the same password as the previous one.', tag.b(u' Please choose a different password.'), style='color:chocolate')
			return template, data, None
		dn = 'uid=%s,%s,%s' % (req.authname, self.userdn, self.basedn)
		try:
			self.log.warn('LDAP change password on DN %s' % dn)
			self.ldap.passwd_s(dn, old, p1)
		except ldap.LDAPError, e:
			data['accountldap_message'] = tag.center(u'An error occured when changing the password.', tag.b(u' Please check the old password and try again.'), style='color:chocolate')
			self.log.warn('LDAP change password. %s' % e)
			return template, data, None
		data['accountldap_message'] = tag.center(tag.b(u'Password has been changed.'), style='color:green')
		return template, data, None

	#	ITemplateProvider interface
	def get_templates_dirs(self):
		from pkg_resources import resource_filename
		return [resource_filename(__name__, 'templates')]

	#	Helper methods
	def _getUserAttributes(self, uid):
		#	Returns full name and email set in LDAP
		filter = 'uid=%s' % uid
		result = []
		try:
			id = self.ldap.search(self.basedn, ldap.SCOPE_SUBTREE, filter, None)
			type, data = self.ldap.result(id, 0)
		except ldap.LDAPError, e:
			self.log.error('LDAP search error. Check your [ldap] options.')
			return ('', '')
		self.log.info('%s - %s' % (data[0][1]['cn'][0], data[0][1]['mail'][0]))
		return (data[0][1]['cn'][0], data[0][1]['mail'][0])